# -*- coding: utf-8 -*-
# cython: language_level=3
# Copyright (c) 2022 Crisp Crow
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
"""Exceptions that can be thrown by this library."""

from __future__ import annotations

__all__: typing.Sequence[str] = ('OnewotError', 'EntityNotFound', 'UserNotFound')

import typing

import attr


@attr.define(slots=True, frozen=True, auto_exc=True)
class OnewotError(RuntimeError):
    """Base for an error raised by this API."""

    code: str = attr.field()
    message: str = attr.field()
    field: str = attr.field()
    value: typing.Union[str, int] = attr.field()


@attr.define(auto_exc=True)
class EntityNotFound(OnewotError):
    """Exception thrown when searched entity has invalid ID or name."""

    field: str = attr.field(init=False)


@attr.define(slots=True, frozen=True, auto_exc=True)
class UserNotFound(EntityNotFound):
    """Exception thrown when searched user has invalid ID or name."""



# -*- coding: utf-8 -*-
# cython: language_level=3
# Copyright (c) 2022 Crisp Crow
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
"""Entities that are used to describe users on WotBlitz."""

from __future__ import annotations

__all__: typing.Sequence[str] = ('User', 'ClanMember')

import abc
import typing
import enum

import attr

if typing.TYPE_CHECKING:
    from onewot.internal import unix
    from onewot import clans
    from onewot import snowflakes


@attr.define(slots=True, frozen=True)
class BaseUser(abc.ABC):
    """Basic interface of user information."""

    id: snowflakes.Snowflake = attr.field()


@attr.define(slots=True, frozen=True)
class ClanMember(BaseUser):
    """Interface of clan member information."""

    clan_id: snowflakes.Snowflake = attr.field()
    joined_at: unix.UnixTime = attr.field()
    role: MemberClanRole = attr.field()
    clan: clans.BaseClan = attr.field()


class MemberClanRole(enum.Enum):
    """Member clan roles interface."""

    PRIVATE: typing.Final[str] = 'private'
    COMMANDER: typing.Final[str] = 'commander'
    DIPLOMAT: typing.Final[str] = 'diplomat'
    VICE_LEADER: typing.Final[str] = 'vice_leader'
    LEADER: typing.Final[str] = 'leader'
    RECRUITER: typing.Final[str] = 'recruiter'
    TREASURER: typing.Final[str] = 'treasurer'

    def __repr__(self) -> str:
        return self.value


@attr.define(slots=True, frozen=True)
class User(BaseUser):
    """Interface of user information."""

    nickname: str = attr.field()
    created_at: unix.UnixTime = attr.field()
    last_battle_time: typing.Optional[unix.UnixTime] = attr.field()
    updated_at: unix.UnixTime = attr.field()
    spotted: int = attr.field()
    max_frags_tank_id: int = attr.field()
    hits: int = attr.field()
    frags: int = attr.field()
    max_xp: int = attr.field()
    max_xp_tank_id: int = attr.field()
    wins: int = attr.field()
    losses: int = attr.field()
    capture_points: int = attr.field()
    battles: int = attr.field()
    damage_dealt: int = attr.field()
    damage_received: int = attr.field()
    max_frags: int = attr.field()
    shots: int = attr.field()
    frags8p: int = attr.field()
    xp: int = attr.field()
    win_and_survived: int = attr.field()
    survived_battles: int = attr.field()
    dropped_capture_points: int = attr.field()
    win_percent: float = attr.field()
    achievements: typing.Type[object] = attr.field()



# -*- coding: utf-8 -*-
# cython: language_level=3
# Copyright (c) 2022 Crisp Crow
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
"""Implementation of an error handler for WotBlitz API"""

from __future__ import annotations

__all__: typing.Sequence[str] = ('ErrorHandlerImpl',)

import typing

from onewot.api import error_handlers
from onewot import errors

if typing.TYPE_CHECKING:
    from onewot.internal import data_binding


class ErrorHandlerImpl(error_handlers.ErrorHandler):
    """Implementation of an error handler for WotBlitz API responses."""

    __slots__: typing.Sequence[str] = ()

    def handle(
        self,
        payload: data_binding.JSONObject,
        params: dict[str, typing.Any]
    ) -> typing.Union[data_binding.JSONObject, errors.OnewotError]:
        if payload['status'] == 'error':
            return errors.OnewotError(**payload['error'])

        if not len(payload['data']):
            return errors.OnewotError('407', 'INVALID_FIELDS', 'fields', params['search'])

        return payload



# -*- coding: utf-8 -*-
# cython: language_level=3
# Copyright (c) 2022 Crisp Crow
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
"""Interface for an error handler for WotBlitz API"""

from __future__ import annotations

__all__: typing.Sequence[str] = ('ErrorHandler',)

import typing
import abc

if typing.TYPE_CHECKING:
    from onewot.internal import data_binding


class ErrorHandler(abc.ABC):
    """Interface for handling WotBlitz API JSON responses."""

    @abc.abstractmethod
    def handle(
        self,
        payload: data_binding.JSONObject,
        params: dict[str, typing.Any]
    ) -> typing.Union[data_binding.JSONObject, errors.OnewotError]:
        """Handle recieved JSON object.

        Parameters
        ----------
        payload : onewot.internal.data_binding.JSONObject
            The JSON payload to handle.
        params : builtins.dict[builtins.str, typing.Any]
            Parameters of HTTP request body.

        Returns
        -------
        typing.Optional[onewot.internal.data_binding.JSONObject]
            The JSON payload.
        onewot.errors.OnewotError
            if API request response returns JSON payload with status `error`
        """



# -*- coding: utf-8 -*-
# cython: language_level=3
# Copyright (c) 2022 Crisp Crow
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
"""API-wide URLs."""

from __future__ import annotations

__all__: typing.Sequence[str] = (
    'BASE_URL',
    'ACCOUNT_INFO_PATH',
    'ACCOUNT_LIST_PATH',
    'ACCOUNT_ACHIEVEMENTS_PATH',
    'CLAN_LIST_PATH',
    'CLAN_INFO_PATH',
    'CLAN_MEMBER_PATH',
    'TOURNAMENT_LIST_PATH',
    'TOURNAMENT_INFO_PATH',
    'API_METHODS',
    'ApiMethod'
)

import typing
import enum

BASE_URL: typing.Final[str] = 'https://api.wotblitz.ru/wotb'
"""Base path to wotblitz API."""

ACCOUNT_INFO_PATH: typing.Final[str] = BASE_URL + '/account/info/'
"""Path to specified account info API method."""

ACCOUNT_LIST_PATH: typing.Final[str] = BASE_URL + '/account/list/'
"""Path to accounts info API method."""

ACCOUNT_ACHIEVEMENTS_PATH: typing.Final[str] = BASE_URL + '/account/achievements/'
"""Path to specified account achievements API method."""

CLAN_LIST_PATH: typing.Final[str] = BASE_URL + '/clans/list/'
"""Path to clans info API method."""

CLAN_INFO_PATH: typing.Final[str] = BASE_URL + '/clans/info/'
"""Path to specified clan info API method."""

CLAN_MEMBER_PATH: typing.Final[str] = BASE_URL + '/clans/accountinfo/'
"""Path to specified clan member info API method."""

TOURNAMENT_LIST_PATH: typing.Final[str] = BASE_URL + '/tournaments/list/'
"""Path to tournaments info API method."""

TOURNAMENT_INFO_PATH: typing.Final[str] = BASE_URL + '/tournaments/info/'
"""Path to specified tournament info API method."""

API_METHODS: typing.Final[dict[ApiMethod, str]] = {
    'account_info': ACCOUNT_INFO_PATH,
    'account_list': ACCOUNT_LIST_PATH,
    'account_achievements': ACCOUNT_ACHIEVEMENTS_PATH,
    'clan_info': CLAN_INFO_PATH,
    'clan_list': CLAN_LIST_PATH,
    'clan_member': CLAN_MEMBER_PATH,
    'tournament_info': TOURNAMENT_INFO_PATH,
    'tournament_list': TOURNAMENT_LIST_PATH
}
"""Dict of API methods."""


class ApiMethod(str, enum.Enum):
    """Enum class with api methods for type hinting."""

    ACCOUNT_INFO: typing.Final[str] = 'account_info'
    ACCOUNT_LIST: typing.Final[str] = 'account_list'
    ACCOUNT_ACHIEVEMENTS: typing.Final[str] = 'account_achievements'
    CLAN_INFO: typing.Final[str] = 'clan_info'
    CLAN_LIST: typing.Final[str] = 'clan_list'
    CLAN_MEMBER: typing.Final[str] = 'clan_member'
    TOURNAMENT_INFO: typing.Final[str] = 'tournament_info'
    TOURNAMENT_LIST: typing.Final[str] = 'tournament_list'



# -*- coding: utf-8 -*-
# cython: language_level=3
# Copyright (c) 2022 Crisp Crow
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
"""Entities that are used to describe tournaments on WotBlitz."""

from __future__ import annotations

__all__: typing.Sequence[str] = ('Tournament',)

import attr
import typing
import enum

if typing.TYPE_CHECKING:
    from onewot.internal import unix
    from onewot import snowflakes


@attr.define(slots=True, frozen=True)
class Tournament:
    """Interface of tournament information."""

    id: snowflakes.Snowflake = attr.field()
    description: str = attr.field()
    end_at: unix.UnixTime = attr.field()
    matches_start_at: unix.UnixTime = attr.field()
    registration_end_at: unix.UnixTime = attr.field()
    registration_start_at: unix.UnixTime = attr.field()
    start_at: unix.UnixTime = attr.field()
    status: TournamentStatus = attr.field()
    title: str = attr.field()
    award: TournamentAward = attr.field()
    fee: TournamentFee = attr.field()
    logo: TournamentLogo = attr.field()
    winner_award: TournamentWinnerAward = attr.field()


@attr.define(slots=True, frozen=True)
class TournamentEconomy:
    """Interface of tournament economy information."""

    amount: int = attr.field()
    currency: CurrencyType = attr.field()


@attr.define(slots=True, frozen=True)
class TournamentAward(TournamentEconomy):
    """Interface of tournament award information."""


@attr.define(slots=True, frozen=True)
class TournamentFee(TournamentEconomy):
    """Interface of tournament fee information."""


@attr.define(slots=True, frozen=True)
class TournamentWinnerAward(TournamentEconomy):
    """Interface of tournament winner award information."""


@attr.define(slots=True, frozen=True)
class TournamentLogo:
    """Interface of tournament logo information."""

    original: str = attr.field()
    preview: str = attr.field()


class TournamentStatus(enum.Enum):
    """Interface of tournament status information."""

    UPCOMING: typing.Final[str] = 'upcoming'
    FINISHED: typing.Final[str] = 'finished'
    RUNNING: typing.Final[str] = 'running'
    REGISTRATION_STARTED: typing.Final[str] = 'registration_started'

    def __repr__(self) -> str:
        return self.value


class CurrencyType(enum.Enum):
    """Interface of tournament currency type information."""

    GOLD: typing.Final[str] = 'gold'
    CREDITS: typing.Final[str] = 'credits'
    FREE_EXPERIENCE: typing.Final[str] = 'free_experience'

    def __repr__(self) -> str:
        return self.value



# -*- coding: utf-8 -*-
# cython: language_level=3
# Copyright (c) 2022 Crisp Crow
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
"""Basic implementation of an entity factory for WotBlitz entities."""

from __future__ import annotations

__all__: typing.Sequence[str] = ('EntityFactoryImpl',)

import typing

from onewot import clans as clan_models
from onewot import users as user_models
from onewot import achievements as achievement_models
from onewot import tournaments as tournament_models
from onewot import snowflakes
from onewot.api import entity_factory
from onewot.internal import unix

if typing.TYPE_CHECKING:
    from onewot.internal import data_binding


def calculate_win_percent(payload: data_binding.JSONObject) -> typing.Union[int, float]:
    battles = payload['statistics']['all']['battles']
    wins = payload['statistics']['all']['wins']

    try:
        percent = wins/battles * 100
    except ZeroDivisionError:
        return 0.0

    return round(percent, 2)


class EntityFactoryImpl(entity_factory.EntityFactory):
    """Standard implementation for a deserializer.
    This will convert objects from JSON compatible representations.
    """

    __slots__: typing.Sequence[str] = ()

    def deserialize_clan(self, payload: data_binding.JSONObject) -> clan_models.Clan:
        return clan_models.Clan(
            id=snowflakes.Snowflake(payload['clan_id']),
            created_at=unix.UnixTime(payload.get('created_at')),
            creator_id=snowflakes.Snowflake(payload.get('creator_id')),
            creator_name=payload.get('creator_name'),
            description=payload.get('description'),
            emblem_set_id=snowflakes.Snowflake(payload.get('emblem_set_id')),
            is_clan_disbanded=payload['is_clan_disbanded'],
            leader_id=snowflakes.Snowflake(payload.get('leader_id')),
            leader_name=payload.get('leader_name'),
            members_count=payload.get('members_count'),
            members_ids=payload.get('members_ids'),
            motto=payload.get('motto'),
            name=payload.get('name'),
            old_name=payload.get('old_name'),
            old_tag=payload.get('old_tag'),
            recruiting_policy=clan_models.RecruitingPolicy(
                payload.get('recruiting_policy')
            ) if not payload['is_clan_disbanded'] else None,
            renamed_at=payload.get('renamed_at'),
            tag=payload.get('tag'),
            updated_at=unix.UnixTime(payload['updated_at']),
            recruiting_options=self.deserialize_recruiting_options(
                payload['recruiting_options']
            )
        )

    def deserialize_user(self, payload: data_binding.JSONObject) -> user_models.User:
        return user_models.User(
            id=snowflakes.Snowflake(payload['account_id']),
            created_at=unix.UnixTime(payload['created_at']),
            last_battle_time=payload.get('last_battle_time'),
            nickname=payload['nickname'],
            updated_at=unix.UnixTime(payload['updated_at']),
            spotted=payload['statistics']['all']['spotted'],
            max_frags_tank_id=payload['statistics']['all']['max_frags_tank_id'],
            hits=payload['statistics']['all']['hits'],
            frags=payload['statistics']['all']['frags'],
            max_xp=payload['statistics']['all']['max_xp'],
            max_xp_tank_id=payload['statistics']['all']['max_xp_tank_id'],
            wins=payload['statistics']['all']['wins'],
            losses=payload['statistics']['all']['losses'],
            capture_points=payload['statistics']['all']['capture_points'],
            battles=payload['statistics']['all']['battles'],
            damage_dealt=payload['statistics']['all']['damage_dealt'],
            damage_received=payload['statistics']['all']['damage_received'],
            max_frags=payload['statistics']['all']['max_frags'],
            shots=payload['statistics']['all']['shots'],
            frags8p=payload['statistics']['all']['frags8p'],
            xp=payload['statistics']['all']['xp'],
            win_and_survived=payload['statistics']['all']['win_and_survived'],
            survived_battles=payload['statistics']['all']['survived_battles'],
            dropped_capture_points=payload['statistics']['all']['dropped_capture_points'],
            win_percent=calculate_win_percent(payload),
            achievements=payload['achievements']
        )

    def deserialize_achievement(self, payload: data_binding.JSONObject) -> typing.Type[type]:
        achievement = achievement_models.BaseAchievement(payload).make_class()
        payload = dict(payload['achievements'], max_series=payload['max_series'])
        return achievement(**payload)

    def deserialize_base_clan(self, payload: data_binding.JSONObject) -> clan_models.BaseClan:
        return clan_models.BaseClan(
            id=snowflakes.Snowflake(payload['clan_id']),
            created_at=unix.UnixTime(payload['created_at']),
            emblem_set_id=snowflakes.Snowflake(payload['emblem_set_id']),
            members_count=payload['members_count'],
            name=payload['name'],
            tag=payload['tag']
        )

    def deserialize_clan_member(self, payload: data_binding.JSONObject) -> user_models.ClanMember:
        return user_models.ClanMember(
            id=snowflakes.Snowflake(payload['account_id']),
            clan_id=snowflakes.Snowflake(payload['clan_id']),
            joined_at=unix.UnixTime(payload['joined_at']),
            role=user_models.MemberClanRole(payload['role']),
            clan=self.deserialize_base_clan(payload['clan']) if 'clan' in payload else None
        )

    def deserialize_recruiting_options(self, payload: data_binding.JSONObject) -> clan_models.RecruitingOptions:
        return clan_models.RecruitingOptions(
            vehicles_level=payload['vehicles_level'],
            wins_ratio=payload['wins_ratio'],
            average_battles_per_day=payload['average_battles_per_day'],
            battles=payload['battles'],
            average_damage=payload['average_damage']
        )

    def deserialize_tournament(self, payload: data_binding.JSONObject) -> tournament_models.Tournament:
        return tournament_models.Tournament(
            id=snowflakes.Snowflake(payload['tournament_id']),
            description=payload['description'],
            end_at=payload['end_at'],
            matches_start_at=unix.UnixTime(payload['matches_start_at']),
            registration_end_at=unix.UnixTime(payload['registration_end_at']),
            registration_start_at=unix.UnixTime(payload['registration_start_at']),
            start_at=unix.UnixTime(payload['start_at']),
            status=tournament_models.TournamentStatus(payload['status']),
            title=payload['title'],
            award=tournament_models.TournamentAward(
                amount=payload['award']['amount'],
                currency=tournament_models.CurrencyType(
                    payload['award']['currency']
                ) if payload['award']['currency'] else None
            ),
            fee=tournament_models.TournamentFee(
                amount=payload['fee']['amount'],
                currency=tournament_models.CurrencyType(
                    payload['fee']['currency']
                ) if payload['fee']['currency'] else None
            ),
            logo=tournament_models.TournamentLogo(**payload['logo']),
            winner_award=tournament_models.TournamentWinnerAward(
                amount=payload['winner_award']['amount'],
                currency=tournament_models.CurrencyType(
                    payload['winner_award']['currency']
                ) if payload['winner_award']['currency'] else None
            )
        )



# -*- coding: utf-8 -*-
# cython: language_level=3
# Copyright (c) 2022 Crisp Crow
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
"""Entities that are used to describe achievements on WotBlitz."""

from __future__ import annotations

__all__: typing.Sequence[str] = ('BaseAchievement',)

import typing

import attr

if typing.TYPE_CHECKING:
    from onewot.internal import data_binding


class BaseAchievement:
    """Class for creating `Achievement` object."""

    def __init__(self, payload: data_binding.JSONObject) -> None:
        self.achievements = dict(
            **payload['achievements'],
            max_series=payload['max_series']
        )

    def make_class(self) -> typing.Type[object]:
        """Create a new class with user achievements data."""

        attrs = list(self.achievements)
        return attr.make_class(
            'Achievement',
            attrs=attrs,
            repr=True
        )



# -*- coding: utf-8 -*-
# cython: language_level=3
# Copyright (c) 2022 Crisp Crow
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
"""Implementation of a HTTP-client for WotBliz API."""

from __future__ import annotations

__all__: typing.Sequence[str] = ('HTTPClientImpl',)

import typing

from requests_futures import sessions

from onewot.impl import entity_factory
from onewot.impl import error_handlers
from onewot.api import http
from onewot.internal import data_binding
from onewot import urls
from onewot import snowflakes
from onewot import errors

if typing.TYPE_CHECKING:
    from onewot import users
    from onewot import clans
    from onewot import tournaments


class HTTPClientImpl(http.HTTPClient):
    """HTTP client for making requests to API methods.
    This manages making HTTP/1.1 requests to the API and using the entity
    factory within the passed application instance to deserialize JSON responses
    to Pythonic data classes that are used throughout this library.
    Parameters
    ----------
    application_id : builtins.str
        Application ID of WotBlitz API application.
    """

    __slots__: typing.Sequence[str] = (
        '_application_id',
        '_session',
        '_error_handler',
        '_entity_factory'
    )

    def __init__(self, application_id: str) -> None:
        self._application_id: str = application_id
        self._session: sessions.FuturesSession = sessions.FuturesSession()
        self._error_handler: error_handlers.ErrorHandlerImpl = error_handlers.ErrorHandlerImpl()
        self._entity_factory: entity_factory.EntityFactoryImpl = entity_factory.EntityFactoryImpl()

    def _request(
        self,
        method: str,
        path: str,
        params: dict[str, typing.Any]
    ) -> typing.Optional[data_binding.JSONObject]:
        payload = self._session.request(method, path, params).result().json()
        handled_payload = self._error_handler.handle(payload, params)

        if isinstance(handled_payload, errors.OnewotError):
            raise handled_payload

        return payload

    def _create_params(self, **kwargs: typing.Any) -> dict[str, typing.Any]:
        return dict(kwargs, application_id=self._application_id)

    def _get_entity_id(self, entity_type: str, entity_name: str) -> snowflakes.Snowflake:
        params = self._create_params(search=entity_name)
        path = self._entity_factory.entities[entity_type]
        payload = self._request('GET', path, params)

        return snowflakes.Snowflake(payload['data'][0][entity_type])

    def _get_payload(
        self,
        entity_id: snowflakes.Snowflake,
        api_method: urls.ApiMethod,
        get_achievements: typing.Optional[bool] = False,
        **params: typing.Any
    ) -> data_binding.JSONObject:
        params = self._create_params(**params)
        payload = self._request('GET', urls.API_METHODS[api_method], params)

        try:
            if get_achievements:
                achievement_payload = self._request('GET', urls.ACCOUNT_ACHIEVEMENTS_PATH, params)
                achievements = self._entity_factory.deserialize_achievement(achievement_payload['data'][str(entity_id)])
                return dict(**payload['data'][str(entity_id)], achievements=achievements)

            return dict(**payload['data'][str(entity_id)])
        except TypeError:
            raise errors.EntityNotFound('404', 'ENTITY_NOT_FOUND', entity_id)

    def fetch_user(self, user: typing.Union[str, snowflakes.Snowflake]) -> users.User:
        if isinstance(user, str):
            user = self._get_entity_id('account_id', user)

        payload = self._get_payload(
            entity_id=user,
            api_method='account_info',
            get_achievements=True,
            account_id=user
        )
        return self._entity_factory.deserialize_user(payload=payload)

    def fetch_clan(self, clan: typing.Union[str, snowflakes.Snowflake]) -> clans.Clan:
        if isinstance(clan, str):
            clan = self._get_entity_id('clan_id', clan)

        payload = self._get_payload(
            entity_id=clan,
            api_method='clan_info',
            clan_id=clan
        )
        return self._entity_factory.deserialize_clan(payload)

    def fetch_user_achievements(self, user: typing.Union[str, snowflakes.Snowflake]) -> typing.Type[object]:
        if isinstance(user, str):
            user = self._get_entity_id('account_id', user)

        payload = self._get_payload(
            entity_id=user,
            api_method='account_achievements',
            account_id=user
        )
        return self._entity_factory.deserialize_achievement(payload)

    def fetch_clan_member(self, user: typing.Union[str, snowflakes.Snowflake]) -> users.ClanMember:
        if isinstance(user, str):
            user = self._get_entity_id('account_id', user)

        payload = self._get_payload(
            entity_id=user,
            api_method='clan_member',
            account_id=user
        )
        return self._entity_factory.deserialize_clan_member(payload)

    def fetch_tournaments(
        self,
        tournament_name: typing.Optional[str] = None,
        page_number: typing.Optional[int] = None,
        limit: typing.Optional[int] = None
    ) -> typing.Optional[tuple[tournaments.Tournament]]:
        params = self._create_params(
            search=tournament_name,
            limit=limit,
            page_no=page_number
        )
        payload = self._request('GET', urls.TOURNAMENT_LIST_PATH, params)
        return tuple(
            self._entity_factory.deserialize_tournament(data) for data in payload['data']
        )

    def fetch_users_by_id(self, user_ids: typing.Iterable[snowflakes.Snowflake]) -> tuple[users.User]:
        return tuple(
            map(
                lambda user_id: self._entity_factory.deserialize_user(
                    payload=self._get_payload(
                        entity_id=user_id,
                        api_method='account_info',
                        get_achievements=True,
                        account_id=user_id
                        ),
                ), user_ids
            )
        )

    def fetch_tournament(self, tournament: typing.Union[str, snowflakes.Snowflake]) -> tournaments.Tournament:
        if isinstance(tournament, str):
            tournament = self._get_entity_id('tournament_id', tournament)

        payload = self._get_payload(
            entity_id=tournament,
            api_method='tournament_info',
            tournament_id=tournament
        )
        return self._entity_factory.deserialize_tournament(payload)



# -*- coding: utf-8 -*-
# cython: language_level=3
# Copyright (c) 2022 Crisp Crow
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
"""WotBlitz client module."""

from __future__ import annotations

__all__: typing.Sequence[str] = ('WOTBClient',)

import typing

from onewot.impl import http

if typing.TYPE_CHECKING:
    from onewot import users
    from onewot import clans
    from onewot import tournaments
    from onewot import snowflakes


class WOTBClient:
    """WotBlitz API client.
    This is the class, you will want to create WotBlitz client.
    Parameters
    ----------
    application_id : builtins.str
        Application ID of WotBlitz API applications.
    Example
    -------
    Setting up client:
    ```py
    import os
    import onewot
    client = onewot.WOTBClient(os.environ["APPLICATION_ID"])
    ```
    """

    __slots__: typing.Sequence[str] = ('_application_id', '_http')

    def __init__(self, application_id: str) -> None:
        self._application_id = application_id
        self._http: http.HTTPClientImpl = http.HTTPClientImpl(application_id)

    def fetch_user(self, user: typing.Union[str, snowflakes.Snowflake]) -> users.User:
        """Fetch a user by name or identificator.
        Parameters
        ----------
        user : typing.Union[builtins.str, snowflakes.Snowflake]
            A user to fetch.
        Returns
        -------
        users.User
            User object.
        """
        return self._http.fetch_user(user)

    def fetch_clan(self, clan: typing.Union[str, snowflakes.Snowflake]) -> clans.Clan:
        """Fetch clan by name or identificator.
        Parameters
        ----------
        clan : typing.Union[builtins.str snowflakes.Snowflake]
            A clan to fetch.
        Returns
        -------
        clans.Clan
            Clan object.
        """
        return self._http.fetch_clan(clan)

    def fetch_clan_member(self, member: typing.Union[str, snowflakes.Snowflake]) -> users.ClanMember:
        """Fetch clan member by name or identificator.
        Parameters
        ----------
        member : typing.Union[builtins.str, snowflakes.Snowflake]
            A clan member to fetch.
        Returns
        -------
        users.ClanMember
            Clan member object
        """
        return self._http.fetch_clan_member(member)

    def fetch_tournaments(
        self,
        tournament_name: typing.Optional[str] = None,
        page_number: typing.Optional[int] = None,
        limit: typing.Optional[int] = None
    ) -> typing.Optional[tuple[tournaments.Tournament]]:
        """Fetch upcoming, finished and running tournaments.
        Parameters
        ----------
        tournament_name : typing.Optional[builtins.str]
            Tournament to fetch. Defaults to `builtins.None`.
        page_number : typing.Optional[builtins.int]
            Page for search. Defaults to `builtins.None`.
        limit : typing.Optional[builtins.int]
            Tournament search limit. Maximum value is 100.
            Defaults to `builtins.None`.
        Returns
        -------
        builtins.tuple[tournaments.Tournament]
            List of tournaments.
        """
        return self._http.fetch_tournaments(limit=limit)

    def fetch_users_by_id(self, user_ids: typing.Iterable[snowflakes.Snowflake]) -> tuple[users.User]:
        """Fetch users by their ID.
        Parameters
        ----------
        user_ids : typing.Iterable[snowflakes.Snowflakes]
            An iterable object of user IDs.
        Returns
        -------
        builtins.tuple[users.User]
            A tuple of user objects.
        """
        return self._http.fetch_users_by_id(user_ids)

    def fetch_tournament(self, tournament: typing.Union[str, snowflakes.Snowflake]) -> tournaments.Tournament:
        """Fetch tournament by name or identificator.
        Parameters
        ----------
        tournament : typing.Union[builtins.str, snowflakes.Snowflake]
            Tournament to fetch.
        Returns
        -------
        tournaments.Tournament
            Deserialized tournament object.
        """
        return self._http.fetch_tournament(tournament)



# -*- coding: utf-8 -*-
# cython: language_level=3
# Copyright (c) 2022 Crisp Crow
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
"""Provides an interface for HTTP-client implementations to follow."""

from __future__ import annotations

__all__: typing.Sequence[str] = ('HTTPClient',)

import typing
import abc

if typing.TYPE_CHECKING:
    from onewot import users
    from onewot import clans
    from onewot import tournaments
    from onewot import snowflakes
    from onewot import urls
    from onewot.internal import data_binding


class HTTPClient(abc.ABC):
    """Interface for functionality that a HTTP-client implementation provides."""

    @abc.abstractmethod
    def _request(
        self,
        method: str,
        path: str,
        params: dict[str, typing.Any]
    ) -> typing.Optional[data_binding.JSONObject]:
        """Method for making HTTP-requests.
        Parameters
        ----------
        method : builins.str
            One of HTTP request methods.
        path : builins.str
            Path to API method.
        params : builtins.dict[builtins.str, typing.Any]
            Dict of specified parameters for request.
        Returns
        -------
        onewot.internal.data_binding.JSONObject
            JSON-object with recieved data.
        """

    @abc.abstractmethod
    def _create_params(self, **kwargs: typing.Any) -> dict[str, typing.Any]:
        """Create parameters for HTTP-request.
        Parameters
        ----------
        kwargs : typing.Any
            Parameters for HTTP request body.
        Returns
        -------
        builtins.dict[builtins.str, typing.Any]
            Dict of specified keyword arguments for request params.
        """

    @abc.abstractmethod
    def _get_entity_id(self, entity_type: str, entity_name: str) -> snowflakes.Snowflake:
        """Fetch entity by his name.
        Parameters
        ----------
        entity_type : builtins.str
            One of entities of entity factory.
        entity_name : builtins.str
            Specified entity name.
        Returns
        -------
        onewot.snowflakes.Snowflake
            Entity identificator.
        """

    @abc.abstractmethod
    def _get_payload(
        self,
        entity_id: snowflakes.Snowflake,
        api_method: urls.ApiMethod,
        get_achievements: typing.Optional[bool] = False,
        **params: typing.Any
    ) -> data_binding.JSONObject:
        """Get payload data of entity from API.
        Parameters
        ----------
        entity_id : snowflakes.Snowflake
            Entity to fetch.
        api_method : typing.Union[urls.API_METHODS]
            Api method to use.
        get_achievements : typing.Optional[builtins.bool]
            Fetch user achievements or not. Defaults to `builtins.False`
        params : typing.Any
            Parameters for HTTP request body.
        Returns
        -------
        onewot.internal.data_binding.JSONObject
            JSON response from API request.
        Raises
        ------
        onewot.errors.EntityNotFound
            If provided entity does not exists.
        """

    @abc.abstractmethod
    def fetch_user(self, user: typing.Union[str, snowflakes.Snowflake]) -> users.User:
        """Fetch user by his ID or name.
        Parameters
        ----------
        user : typing.Union[builtins.str, onewot.snowflakes.Snowflake]
            User to fetch.
        Returns
        -------
        onewot.users.User
            Deserialized user object.
        """

    @abc.abstractmethod
    def fetch_clan(self, clan: typing.Union[str, snowflakes.Snowflake]) -> clans.Clan:
        """Fetch clan by his ID or name.
        Parameters
        ----------
        clan : typing.Union[builtins.str, onewot.snowflakes.Snowflake]
            Clan to fetch.
        Returns
        -------
        onewot.clans.Clan
            Deserialized clan object.
        """

    @abc.abstractmethod
    def fetch_user_achievements(self, user: typing.Union[str, snowflakes.Snowflake]) -> typing.Type[object]:
        """Fetch user achievements.
        Parameters
        ----------
        user : typing.Union[builtins.str, snowflakes.Snowflake]
            User to fetch achievements.
        Returns
        -------
        typing.Type[object]
            Deserialized achievement object.
        """

    @abc.abstractmethod
    def fetch_clan_member(self, user: typing.Union[str, snowflakes.Snowflake]) -> users.ClanMember:
        """Fetch clan member by his ID or name.
        Parameters
        ----------
        user : typing.Union[builtins.str, onewot.snowflakes.Snowflake]
            User to fetch.
        Returns
        -------
        onewot.users.ClanMember
            Deserialized clan member object.
        """

    @abc.abstractmethod
    def fetch_tournaments(
        self,
        tournament_name: typing.Optional[str] = None,
        page_number: typing.Optional[int] = None,
        limit: typing.Optional[int] = None
    ) -> typing.Optional[tuple[tournaments.Tournament]]:
        """Fetch tournaments.
        Parameters
        ----------
        tournament_name : typing.Optional[builtins.str]
            Tournament to fetch. Defaults to `builtins.None`.
        page_number : typing.Optional[builtins.int]
            Page for search. Defaults to `builtins.None`.
        limit : typing.Optional[builtins.int]
            Tournament search limit. Defaults to `builtins.None`.
        Returns
        -------
        typing.Optional[builtins.tuple[onewot.tournaments.Tournament]]
            Deserialized tournament objects. If no tournaments found,
            returns empty `builtins.tuple`.
        """

    @abc.abstractmethod
    def fetch_users_by_id(self, user_ids: typing.Iterable[snowflakes.Snowflake]) -> tuple[users.User]:
        """Fetch users by their ID.
        Parameters
        ----------
        user_ids : typing.Iterable[onewot.snowflake.Snowflake]
            An iterable object of user IDs.
        Returns
        -------
        builtins.tuple[onewot.users.User]
            Deserialized user objects.
        """

    @abc.abstractmethod
    def fetch_tournament(self, tournament: typing.Union[str, snowflakes.Snowflake]) -> tournaments.Tournament:
        """Fetch tournament by his ID or name.
        Parameters
        ----------
        tournament : typing.Union[builtins.str, onewot.snowflakes.Snowflake]
            Tournament to fetch.
        Returns
        -------
        onewot.tournaments.Tournament
            Deserialized tournament object.
        """



# -*- coding: utf-8 -*-
# cython: language_level=3
# Copyright (c) 2022 Crisp Crow
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
"""Core interface for an object that deserializes API objects."""

from __future__ import annotations

__all__: typing.Sequence[str] = ('EntityFactory',)

import typing
import abc

from onewot import urls

if typing.TYPE_CHECKING:
    from onewot import tournaments as tournament_models
    from onewot import clans as clan_models
    from onewot import users as user_models
    from onewot import achievements as achievement_models
    from onewot.internal import data_binding


class EntityFactory(abc.ABC):
    """Interface for components that deserialize JSON payloads."""

    @property
    def entities(self) -> dict[str, str]:
        return {
            'account_id': urls.ACCOUNT_LIST_PATH,
            'clan_id': urls.CLAN_LIST_PATH,
            'tournament_id': urls.TOURNAMENT_LIST_PATH
        }

    @abc.abstractmethod
    def deserialize_clan(self, payload: data_binding.JSONObject) -> clan_models.Clan:
        """Parse a raw payload from WotBlitz into a clan object.
        Parameters
        ----------
        payload : onewot.internal.data_binding.JSONObject
            The JSON payload to deserialize.
        Returns
        -------
        onewot.clans.Clan
            The deserialized clan information object.
        """

    @abc.abstractmethod
    def deserialize_user(self, payload: data_binding.JSONObject) -> user_models.User:
        """Parse a raw payload from WotBlitz into a user object.
        Parameters
        ----------
        payload : onewot.internal.data_binding.JSONObject
            The JSON payload to deserialize.
        Returns
        -------
        onewot.users.User
            The deserialized user information object.
        """

    @abc.abstractmethod
    def deserialize_achievement(self, payload: data_binding.JSONObject) -> achievement_models.Achievement:
        """Parse a raw payload from WotBlitz into an achivement object.
        Parameters
        ----------
        payload : onewot.internal.data_binding.JSONObject
            The JSON payload to deserialize.
        Returns
        -------
        onewot.achivements.Achievement
            The deserialized user achivements information object.
        """

    @abc.abstractmethod
    def deserialize_base_clan(self, payload: data_binding.JSONObject) -> clan_models.BaseClan:
        """Parse a raw payload from WotBlitz into a base clan object.
        Parameters
        ----------
        payload : onewot.internal.data_binding.JSONObject
            The JSON payload to deserialize.
        Returns
        -------
        onewot.clans.BaseClan
            The deserialized base clan information object.
        """

    @abc.abstractmethod
    def deserialize_clan_member(self, payload: data_binding.JSONObject) -> user_models.ClanMember:
        """Parse a raw payload from WotBlitz into a clan member object.
        Parameters
        ----------
        payload : onewot.internal.data_binding.JSONObject
            The JSON payload to deserialize.
        Returns
        -------
        onewot.users.ClanMember
            The deserialized clan member information object.
        """

    @abc.abstractmethod
    def deserialize_recruiting_options(self, payload: data_binding.JSONObject) -> clan_models.RecruitingOptions:
        """Parse a raw payload from WotBlitz into a recruiting clan options object.
        Parameters
        ----------
        payload : onewot.internal.data_binding.JSONObject
            The JSON payload to deserialize.
        Returns
        -------
        onewot.clans.RecruitingOptions
            The deserialized recruiting options information object.
        """

    @abc.abstractmethod
    def deserialize_tournament(self, payload: data_binding.JSONObject) -> tournament_models.Tournament:
        """Parse a raw payload from WotBlitz into a tournament object.
        Parameters
        ----------
        payload : onewot.internal.data_binding.JSONObject
            The JSON payload to deserialize.
        Returns
        -------
        onewot.tournaments.Tournament
            The deserialized tournament information object.
        """



# Copyright (c) 2022 Crisp Crow
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import os
import re
import types

import setuptools


def long_description():
    with open('README.md') as file:
        return file.read()


def parse_meta():
    with open(os.path.join('onewot', '_about.py')) as fp:
        code = fp.read()

    token_pattern = re.compile(r"^__(?P<key>\w+)?__.*=\s*(?P<quote>(?:'{3}|\"{3}|'|\"))(?P<value>.*?)(?P=quote)", re.M)

    groups = {}

    for match in token_pattern.finditer(code):
        group = match.groupdict()
        groups[group['key']] = group['value']

    return types.SimpleNamespace(**groups)


def parse_requirements_file(path):
    with open(path) as file:
        dependencies = (d.strip() for d in file.read().split('\n') if d.strip())
        return [d for d in dependencies if not d.startswith('#')]


metadata = parse_meta()

setuptools.setup(
    name=metadata.name,
    version=metadata.version,
    description=metadata.description,
    long_description=long_description(),
    long_description_content_type='text/markdown',
    author=metadata.author,
    author_email=metadata.email,
    license=metadata.license,
    url=metadata.url,
    python_requires='>=3.6',
    packages=setuptools.find_namespace_packages(include=['onewot*']),
    install_requires=parse_requirements_file('requirements.txt'),
    include_package_data=True,
    zip_safe=False,
    project_urls={
        'Source (GitHub)': metadata.url,
        'Issue Tracker': metadata.issue_tracker,
    },
    classifiers=[
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: Implementation :: CPython',
    'Natural Language :: English',
    'License :: OSI Approved :: MIT License',
    'Operating System :: OS Independent'
    ]
)



# -*- coding: utf-8 -*-
# cython: language_level=3
# Copyright (c) 2022 Crisp Crow
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
"""Package metadata."""

import typing


__name__: typing.Final[str] = 'onewot'
__author__: typing.Final[str] = 'Crisp Crow'
__description__: typing.Final[str] = 'The WotBlitz API wrapper for Python3'
__copyright__: typing.Final[str] = '© 2022-present Crisp Crow'
__email__: typing.Final[str] = 'minikslyh@gmail.com'
__license__: typing.Final[str] = 'MIT'
__url__: typing.Final[str] = 'https://github.com/CrispCrow/Onewot'
__issue_tracker__: typing.Final[str] = 'https://github.com/CrispCrow/Onewot/issues'
__version__: typing.Final[str] = '0.3'



# -*- coding: utf-8 -*-
# cython: language_level=3
# Copyright (c) 2022 Crisp Crow
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
"""Entities that are used to describe clans on WotBlitz."""

from __future__ import annotations

__all__: typing.Sequence[str] = ('Clan', 'BaseClan', 'RecruitingOptions', 'RecruitingPolicy')

import abc
import typing
import enum

import attr

if typing.TYPE_CHECKING:
    from onewot.internal import unix
    from onewot import snowflakes


@attr.define(slots=True, frozen=True)
class BaseClan(abc.ABC):
    """Basic interface of clan information."""

    id: snowflakes.Snowflake = attr.field()
    created_at: typing.Optional[unix.UnixTime] = attr.field()
    emblem_set_id: typing.Optional[snowflakes.Snowflake] = attr.field()
    members_count: typing.Optional[int] = attr.field()
    name: typing.Optional[str] = attr.field()
    tag: typing.Optional[str] = attr.field()


@attr.define(slots=True, frozen=True)
class Clan(BaseClan):
    """Interface of clan information."""

    creator_id: typing.Optional[snowflakes.Snowflake] = attr.field()
    creator_name: typing.Optional[str] = attr.field()
    description: typing.Optional[str] = attr.field()
    is_clan_disbanded: bool = attr.field()
    leader_id: typing.Optional[snowflakes.Snowflake] = attr.field()
    leader_name: typing.Optional[str] = attr.field()
    members_ids: typing.Optional[list[snowflakes.Snowflake]] = attr.field()
    motto: typing.Optional[str] = attr.field()
    old_name: typing.Optional[str] = attr.field()
    old_tag: typing.Optional[str] = attr.field()
    recruiting_policy: typing.Optional[RecruitingPolicy] = attr.field()
    renamed_at: typing.Optional[unix.UnixTime] = attr.field()
    updated_at: typing.Optional[unix.UnixTime] = attr.field()
    recruiting_options: typing.Optional[RecruitingOptions] = attr.field()


@attr.define(slots=True, frozen=True)
class RecruitingOptions:
    """Interface of clan recruiting optins information."""

    vehicles_level: int = attr.field()
    wins_ratio: int = attr.field()
    average_battles_per_day: int = attr.field()
    battles: int = attr.field()
    average_damage: int = attr.field()


class RecruitingPolicy(enum.Enum):
    """Recruiting policy of clan."""

    OPEN: typing.Final[str] = 'open'
    RESTRICTED: typing.Final[str] = 'restricted'

    def __repr__(self) -> str:
        return self.value
